@@ -64,7 +64,10 @@ class FormComposer:
     opt_tag_help: str = 'small'
     """Tag to be used for hints."""
 
-    opt_tag_feedback: str = 'span'
+    opt_tag_feedback: str = 'div'
+    """Tag to be used for feedback."""
+
+    opt_tag_feedback_line: str = 'div'
     """Tag to be used for feedback."""
 
     opt_submit: str = _('Submit')
@@ -82,7 +85,11 @@ class FormComposer:
     """Attributes to apply to hints."""
 
     attrs_feedback: TypeAttrs = None
-    """Attributes to apply to feedback (validation notes)."""
+    """Attributes to apply to feedback (validation notes).
+    
+    FORM macros here denotes global (non-field) form feedback attrs.
+    
+    """
 
     groups: Dict[str, str] = None
     """Map alias to group titles."""
@@ -107,6 +114,7 @@ class FormComposer:
     def __init__(self, form: Union['SiteformsMixin', Form]):
         self.form = form
         self.groups = self.groups or {}
+        self.attrs_feedback = self.attrs_feedback or {}
 
     def __init_subclass__(cls) -> None:
         # Implements attributes enrichment - inherits attrs values from parents.
@@ -236,22 +244,43 @@ class FormComposer:
         return f'{label}'
 
     def _format_feedback_lines(self, errors: List) -> str:
-        return '\n'.join([f'<div>{error}</div>' for error in errors])
+        tag = self.opt_tag_feedback_line
+        return '\n'.join([f'<{tag}>{error}</{tag}>' for error in errors])
 
     def _render_feedback(self, field: BoundField) -> str:
 
-        if not self.form.is_submitted:
+        form = self.form
+
+        if not form.is_submitted:
             return ''
 
         errors = field.errors
         if not errors:
             return ''
 
+        if field.is_hidden:
+            # Gather hidden field errors into non-field group.
+            for error in errors:
+                form.add_error(
+                    None,
+                    _('Hidden field "%(name)s": %(error)s') %
+                    {'name': field.name, 'error': str(error)})
+            return ''
+
         attrs = self._attrs_get_basic(self.attrs_feedback, field)
         tag = self.opt_tag_feedback
 
         return f'<{tag} {flatatt(attrs)}>{self._format_feedback_lines(errors)}</{tag}>'
 
+    def _render_feedback_nonfield(self) -> str:
+        errors = self.form.non_field_errors()
+        if not errors:
+            return ''
+
+        attrs = self.attrs_feedback.get(FORM, {})
+        tag = self.opt_tag_feedback
+        return f'<{tag} {flatatt(attrs)}>{self._format_feedback_lines(errors)}</{tag}>'
+
     def _render_help(self, field: BoundField) -> str:
         help_text = field.help_text
 
@@ -285,6 +314,7 @@ class FormComposer:
     def _render_field_box(self, field: BoundField) -> str:
 
         if field.is_hidden:
+            self._render_feedback(field)
             return str(field)
 
         label = ''
@@ -342,7 +372,9 @@ class FormComposer:
 
         form_layout = self.layout[FORM]
 
-        out = []
+        out = [
+            self._render_feedback_nonfield(),
+        ]
 
         if isinstance(form_layout, str):
             # Simple layout defined by macros.
@@ -402,16 +434,13 @@ class FormComposer:
         get_attr = self._attrs_get
         return self._format_value(
             get_attr(self.wrappers, SUBMIT),
-            submit=f'<button type="submit" name="{self.opt_submit_name}" {flatatt(get_attr(self.attrs, SUBMIT))}>{self.opt_submit}</button>'
+            submit=f'<button type="submit" name="{self.opt_submit_name}" value="{self.form._submit_value}"'
+                   f'{flatatt(get_attr(self.attrs, SUBMIT))}>{self.opt_submit}</button>'
         )
 
     def render(self) -> str:
         """Renders form to string."""
 
-        # todo global errors where to place?
-        # todo error for subform field is not rendered. maybe translate to nonfield.
-        # errors_global = self.form.non_field_errors()
-
         html = self._render_layout()
 
         if self.opt_render_form:
