@@ -39,9 +39,6 @@ class Bootstrap4(FormComposer):
     opt_feedback_tooltips: bool = False
     """Whether to render feedback in tooltips."""
 
-    opt_tag_feedback: str = 'div'
-    """Tag to be used for feedback."""
-
     _size_mod: Tuple[str, ...] = ('col-form-label', 'form-control', 'input-group')
     _file_cls = {'class': 'form-control-file'}
 
@@ -211,6 +208,7 @@ class Bootstrap4(FormComposer):
     }
 
     attrs_feedback: TypeAttrs = {
+        FORM: {'class': 'alert alert-danger mb-4', 'role': 'alert'},
         ALL_FIELDS: {'class': _get_attr_feedback},
     }
 
