@@ -17,6 +17,24 @@ def nocss_form_html(form_html):
     return partial(form_html, composer=Composer, model=Thing)
 
 
+def test_nonfield_errors(form, request_factory):
+    request = request_factory().get('some?__submit=siteform&some=hmm')
+
+    frm = form(
+        composer=Composer,
+        some=fields.DateField(widget=fields.HiddenInput())
+    )(src='GET', request=request)
+
+    frm.is_valid()
+
+    frm.add_error(None, 'Errr1')
+    frm.add_error(None, 'Errr2')
+
+    form_html = f'{frm}'
+    assert '<div ><div>Errr1</div>\n<div>Errr2</div>' in form_html
+    assert 'Hidden field "some": Enter a valid date.' in form_html
+
+
 def test_nocss_basic(nocss_form_html, form_fixture_match):
 
     thing = Thing()
@@ -28,7 +46,7 @@ def test_nocss_basic(nocss_form_html, form_fixture_match):
 
 
 def test_nocss_validation(nocss_form_html, request_factory):
-    request = request_factory().get('some?__submit=1')
+    request = request_factory().get('some?__submit=siteform')
     html = nocss_form_html(src='GET', request=request)
     assert 'csrfmiddlewaretoken' in html
     assert '<div>This field is required.</div>' in html
@@ -95,15 +113,13 @@ def form_cls(form):
 
         class SubForm1(Form):
 
-            subform_serialize = True
-
             first = fields.CharField(label='some', help_text='some help')
             second = fields.ChoiceField(label='variants', choices={'1': 'one', '2': 'two'}.items())
 
         form_kwargs = dict(
             composer=Composer,
             somefield=fields.CharField(),
-            fchar=fields.CharField(),
+            fchar=fields.CharField(max_length=30),
             subforms={'fchar': SubForm1},
             model=model,
         )
@@ -130,12 +146,17 @@ def test_nocss_subforms(form_cls, request_factory):
         return valid, frm
 
     # Missing field.
-    valid, frm = check_source('some?__submit=1&somefield=bc&fchar-second=2')
+    valid, frm = check_source('some?__submit=siteform&somefield=bc&fchar-second=2')
     assert not valid
-    assert 'field is required.</div></span><small  id="id_fchar-first_help' in f'{frm}'
+    assert 'field is required.</div></div><small  id="id_fchar-first_help' in f'{frm}'
+
+    # Value is too long for subform field.
+    valid, frm = check_source('some?__submit=siteform&somefield=bc&fchar-second=2&fchar-first=valueistoolong')
+    assert not valid
+    assert 'Subform field "fchar": Ensure this value has at most 30' in f'{frm}'
 
     # All is well.
-    valid, frm = check_source('some?__submit=1&somefield=bc&fchar-second=2&fchar-first=op')
+    valid, frm = check_source('some?__submit=siteform&somefield=bc&fchar-second=2&fchar-first=op')
     assert valid
     assert 'field is required' not in f'{frm}'
 
