@@ -5,8 +5,9 @@ from django.forms import ModelForm as _ModelForm, Form as _Form, HiddenInput, Ba
 from django.forms import fields  # noqa
 from django.http import HttpRequest
 from django.utils.safestring import mark_safe
+from django.utils.translation import gettext_lazy as _
 
-from .widgets import SubformWidget  # noqa
+from .widgets import SubformWidget
 
 if False:  # pragma: nocover
     from .composers.base import FormComposer  # noqa
@@ -27,9 +28,6 @@ class SiteformsMixin(BaseForm):
     subforms: Dict[str, Type['SiteformsMixin']] = None
     """Allows sub forms registration. Expects field name to subform class mapping."""
 
-    subform_serialize: bool = False
-    """Whether to serialize/deserialize value for this subform."""
-
     Composer: Type['FormComposer'] = None
 
     def __init__(
@@ -53,6 +51,9 @@ class SiteformsMixin(BaseForm):
         self.subforms = kwargs.pop('subforms', self.subforms) or {}
         self._subforms: Dict[str, 'SiteformsMixin'] = {}
 
+        # Allow subform using the same submit value as the base form.
+        self._submit_value = kwargs.pop('submit_value', kwargs.get('prefix', self.prefix) or 'siteform')
+
         self._initialize_pre(kwargs)
 
         super().__init__(*args, **kwargs)
@@ -65,18 +66,16 @@ class SiteformsMixin(BaseForm):
         src = self.src
         request = self.request
 
-        is_submitted = False
-
         if src and request:
             data = getattr(request, src)
-            is_submitted = self.Composer.opt_submit_name in data
+            is_submitted = data.get(self.Composer.opt_submit_name, '') == self._submit_value
 
             self.is_submitted = is_submitted
 
             if is_submitted and request.method == src:
                 kwargs['data'] = data
 
-        self._initialize_subforms(is_submitted, kwargs)
+        self._initialize_subforms(kwargs)
 
     def _initialize_post(self):
         # Attach files automatically.
@@ -91,28 +90,30 @@ class SiteformsMixin(BaseForm):
 
     def set_subform_value(self, value: Union[dict, str]):
         """Sets value for subform."""
-        if self.subform_serialize:
+        if isinstance(value, str):
             value = json.loads(value)
         self.initial = value
 
-    def get_subform_value(self) -> Union[dict, str]:
-        """Returns data for subform widget. Default: dict.
-
-        Override to customize returned value.
-
-        """
+    def get_subform_value(self) -> dict:
+        """Returns data dict for subform widget."""
         value = self.cleaned_data
 
-        if self.subform_serialize:
+        if isinstance(value, dict):
             value = json.dumps(value)
 
         return value
 
-    def _initialize_subforms(self, is_submitted, kwargs):
+    def _initialize_subforms(self, kwargs):
         kwargs = kwargs.copy()
         kwargs.pop('instance', None)
 
-        siteforms = {}
+        kwargs.update({
+            'src': self.src,
+            'request': self.request,
+            'submit_value': self._submit_value,
+        })
+
+        subforms = {}
 
         for field_name, subform in self.subforms.items():
 
@@ -128,18 +129,27 @@ class SiteformsMixin(BaseForm):
 
             # Instantiate subform classes with the same arguments.
             sub = subform(**{**kwargs, 'prefix': field_name})
-            sub.is_submitted = is_submitted
-            siteforms[field_name] = sub
+            subforms[field_name] = sub
 
             field.widget = SubformWidget(subform=sub)
 
-        self._subforms = siteforms
+        self._subforms = subforms
 
     def is_valid(self):
         valid = super().is_valid()
 
-        for subform in self._subforms.values():
-            valid &= subform.is_valid()
+        errors = self.errors
+
+        for fieldname, subform in self._subforms.items():
+            subform_valid = subform.is_valid()
+            valid &= subform_valid
+
+            if subform_valid:
+                for error in errors.get(fieldname, []):
+                    self.add_error(
+                        None,
+                        _('Subform field "%(name)s": %(error)s') %
+                        {'name': fieldname, 'error': str(error)})
 
         return valid
 
